/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_CCONFIG_H
#define LINKTEST_CCONFIG_H

// C Header needed since this config file is used for linktest-report 
#include <string.h>
#include <stdint.h>

/* for nanosleep() */
#ifndef _POSIX_C_SOURCE
    #define _POSIX_C_SOURCE 199309L
#endif

/* for gethostname() */
#ifndef _DEFAULT_SOURCE
    #define _DEFAULT_SOURCE 1
#endif

#ifndef _POSIX_HOST_NAME_MAX
    #define _POSIX_HOST_NAME_MAX 4096 //Hopefully enough
#endif

#define LINKTEST_VERSION 2
#define LINKTEST_VERSION_SUB 1
#define LINKTEST_VERSION_PATCHLEVEL 19
#define GIT_HASH_LEN 41
#ifndef GIT_HASH
    #define GIT_HASH "UNKNOWN"
#endif
#define GIT_HASH_SHORT_LEN 8
#ifndef GIT_HASH_SHORT
    #define GIT_HASH_SHORT "UNKNOWN"
#endif

/* Common prefix for all LinkTest environment variables */
#define LINKTEST_ENVIRON_PREFIX "LINKTEST"


/* Linktest identification string for output files (offset: 0x00 for LinkTest data) */
#define LINKTEST_ID "LinkTest"

/* Linktest alignment string for output files. Marks the end of the Binary Header */
#define END_HEADER "END_HEADER"

/* Linktest alignment string for output files. Marks end of thread output */
#define END_BLOCK "END_BLOCK"

/* Seven dimensions is the maximum dimensionality as of this
 * writing (2014-05-17). The K computer uses a 6-D torus,
 * BG/Q uses a 5-D torus. Considering the core affinity/thread
 * affinity as an additional dimension we end up with 7 or
 * 6 dimensions, respectively.
 * Note that 1 + 15 == 16 so there will be no additional padding
 * in mapping_info (not that it actually matters, though).
 */
#define LINKTEST_MAPPING_MAX_DIM 15
/*  Maximal length of the location string. */
#define LINKTEST_MAPPING_MAX_LOCATION_LEN 256
/* Upper limit for the buffer size that is sent inline with verbs */
#define LINKTEST_IBVERBS_MAX_INLINE_SZ 64
/* Upper limit for the buffer size for the date and time when writing
 * started by rank 0. 16 characters are required to store day, month,
 * hour, minute and second. At least 4 for the year are needed if we
 * continue to use the Gregorian calendar or a similar calendar with
 * the same epoch. 32 charcter as such should be sufficient.
 */
#define LINKTEST_OUTPUT_DATETIME_LENGTH 32

#endif
