/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_CMDLINE_H
#define LINKTEST_CMDLINE_H

#include <string>
#include "vcluster.h"
#include "memory.h"

/* Command line arguments. Default values are specified
 * in the .c file.
 *
 * linktest_args is a singleton. Use parse_cmdline_args to
 * get a pointer to the one instance of linktest_args.
 */
struct linktest_args {
    bool          special_args;
    std::int8_t   randomize_buffers;
    std::int8_t   check_buffers;
    std::int8_t   do_alltoall;
    std::int8_t   do_bidir;
    std::int8_t   do_unidir;
    std::int8_t   do_mix;
    std::int8_t   do_serial;
    std::int8_t   do_nosion;
    std::int8_t   do_sion_par;
    std::int8_t   do_use_gpus;
    std::int8_t   do_bisection;
    std::int8_t   use_multi_buf;
    std::int8_t   do_group_processes_by_hostname;
    std::uint64_t num_msg;
    std::uint64_t num_warmup_msg;
    std::uint64_t len_msg;
    std::uint64_t buf_mt_seed;
    std::uint64_t num_multi_buf;
    std::uint64_t num_randomize_tasks;
    std::uint64_t seed_randomize_tasks;
    std::uint64_t seed_randomize_steps;
    std::uint64_t max_stest;
    std::uint64_t min_iterations;
    double        min_walltime;
    static const std::size_t   VClusterImplmaxCharacters=16;
    Alloc::Type alloc_typ;
    std::string   virtual_cluster_implementation;
    std::string   memory_buffer_allocator;
    std::vector<bool> use_gpus_on_task;
    std::string   output;
};

/* Parse the command line arguments. This is usually only done by
 * the root process. If the spawner forwards the command line arguments
 * to all processing elements this function may also be executed
 * everywhere.
 *
 * An earlier design of the command line parsing functionality provided
 * options to pass command line arguments to the vcluster implementation.
 * This however is only feasible if all processes have a copy of the
 * argv argument which is not even guaranteed by the MPI standard.
 * Instead it is better to provide options to the spawner which can use
 * environment variables to pass the information to the processes.
 */

const struct linktest_args* parse_cmdline_args(int argc, char **argv);
const struct linktest_args* fixup_cmdline_args(std::uint64_t size);


/* Broadcast the command line arguments
 */
const struct linktest_args *bcast_cmdline_args(VirtualCluster* cl,
                                               int root, const struct
                                               linktest_args* args);

/* Set values that depend on the cluster rank */
struct linktest_args *localize_cmdline_args(int rank, struct linktest_args* args);

/* Print the command line arguments. The function takes into account the
 * standard/global arguments (see struct linktest_args) as well as the
 * vcluster implementation specific arguments.
 */
void print_cmdline_args(const struct linktest_args *args);

#endif
