
import sys


def _file_as_char_list(fn):
    """
    Read in a file and return the characters in hexadecimal
    string form.
    """
    with open(fn, "rb") as f:
        chars = ["0x%02x" % c for c in f.read()]
    chars += ["0x0"]

    return chars

def _chars_in_groups_of_eight(chars):
    """
    From groups of 8 to print out a long array in multiple lines
    to valid C code.
    """
    sep = ", "
    lines = []

    i = 0
    while i < len(chars):
        line = []
        for j in range(8):
            if (i + j) < len(chars):
                line += [chars[i + j]]

        i = i + 8

        lines += ["\t" + sep.join(line)]

    return lines

def main(i_fn, o_fn, var_name):
    chars = _file_as_char_list(i_fn)
    lines = _chars_in_groups_of_eight(chars)

    with open(o_fn, "w", encoding="utf-8") as f:
        f.write("\n")
        f.write("const unsigned char %s[] = {\n" % var_name)
        f.write(",\n".join(lines) + "\n")
        f.write("};\n")
        f.write("\n")

if "__main__" == __name__:
    main(i_fn=sys.argv[1], o_fn=sys.argv[2], var_name=sys.argv[3])
