/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "environ.h"
#include "config.h"
#include "compiler.h"

#include <cstdlib>

int read_environ_int(const char* name, int *x)
{
    const char* str;
    int tmp;
    char* tailptr;

    str = getenv(name);
    if (unlikely(!str))
        return ERROR;

    tmp = (int )strtol(str, &tailptr, 0);
    if (unlikely(0 != *tailptr)) {
        return ERROR;
    }

    *x = tmp;
    return SUCCESS;
}

int read_environ_long(const char* name, long *x)
{
    const char* str;
    long tmp;
    char* tailptr;

    str = getenv(name);
    if (unlikely(!str))
        return ERROR;

    tmp = (long )strtol(str, &tailptr, 0);
    if (unlikely(0 != *tailptr)) {
        return ERROR;
    }

    *x = tmp;
    return SUCCESS;
}

int read_environ_float(const char* name, float *x)
{
    const char* str;
    float tmp;
    char* tailptr;

    str = getenv(name);
    if (unlikely(!str))
        return ERROR;

    tmp = strtof(str, &tailptr);
    if (unlikely(0 != *tailptr)) {
        return ERROR;
    }

    *x = tmp;
    return SUCCESS;
}

int read_environ_double(const char* name, double *x)
{
    const char* str;
    double tmp;
    char* tailptr;

    str = getenv(name);
    if (unlikely(!str))
        return ERROR;

    tmp = strtod(str, &tailptr);
    if (unlikely(0 != *tailptr)) {
        return ERROR;
    }

    *x = tmp;
    return SUCCESS;
}

int read_environ_str(const char* name, const char **x)
{
    const char* str;

    str = getenv(name);
    if (unlikely(!str))
        return ERROR;

    *x = str;
    return SUCCESS;
}
