/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "config.h"
#include "compiler.h"
#include "error.h"
#include "ibverbs_cq.h"
#include "ibverbs_ctx.h"

#include <utility>
#include <stdexcept>


namespace linktest
{
namespace ibverbs
{

CompletionQueue::CompletionQueue(Context* ctx, int cqe)
: ctx_(ctx), cq(nullptr)
{
    cq = ibv_create_cq(ctx_->ctx, cqe, nullptr, nullptr, 0);
    if (unlikely(!cq)) {
        error("ibv_create_cq() failed");
        throw std::runtime_error("ibv_create_cq() failed");
    }
}

CompletionQueue::~CompletionQueue()
{
    auto ret = ibv_destroy_cq(cq);
    if (unlikely(ret)) {
        error("ibv_destroy_cq() failed");
    }
}

int CompletionQueue::poll(int n, struct ibv_wc *wc)
{
    auto ret = ibv_poll_cq(cq, n, wc);
    if (unlikely(ret < 0)) {
        error("ibv_poll_cq() failed.");
        return ret;
    }

    return ret;
}

}
}
