/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "config.h"
#include "compiler.h"
#include "error.h"
#include "ibverbs_ctx.h"
#include "ibverbs_mr.h"
#include "ibverbs_pd.h"

#include <stdexcept>


namespace linktest
{
namespace ibverbs
{

MemoryRegion::MemoryRegion(MemoryBuffer& buf, ProtectionDomain* pd)
: MemoryBuffer(buf.p(), buf.len(), buf.address_space_id()), 
  pd_(pd), 
  mr_(nullptr), 
  key_(0)
{
    mr_ = ibv_reg_mr(pd_->pd, pointer<void>(), len(),
                IBV_ACCESS_LOCAL_WRITE |
                IBV_ACCESS_REMOTE_WRITE);
    if (unlikely(!mr_)) {
        error("ibv_reg_mr failed.");
        throw std::runtime_error("ibv_reg_mr failed.");
    }

    key_ = mr_->lkey;
}

MemoryRegion::~MemoryRegion()
{
    if (mr_) {
        auto err = ibv_dereg_mr(mr_);
        if (unlikely(err)) {
            warn("ibv_dereg_mr failed.");
        }
    }
}

MemoryRegion::MemoryRegion(RemoteMemoryInfo& info, ProtectionDomain* pd)
: MemoryBuffer(info.p, info.len, AddressSpace::to_remote(info.addr_space_id)),
  pd_(pd), 
  mr_(nullptr), 
  key_(info.key)
{
}

}
}
