/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "memory_cuda.h"
#include "config.h"
#include "error.h"
#include "compiler.h"
#include <cstdio>
#include <cassert>

#include <cuda.h>

namespace linktest
{
namespace cuda
{

#include "cuda_kernels.cc"

template<typename T>
static void verify_fill(T* start, T* end, const T& val)
{
    char buf[8];

    auto err = cuMemcpy(reinterpret_cast<CUdeviceptr>(buf), 
                reinterpret_cast<CUdeviceptr>(start), sizeof(buf));
    if (unlikely(CUDA_SUCCESS != err)) {
        report_cuda_error("cuMemcpy", err);
    }

    for(auto i = 0; i < 8; ++i) {
        assert(buf[i] == val);
    }
}

template<>
void fill<char>(GpuContext* ctx, char* start, char* end, const char& val)
{
    void* args[3] = {
        &start,
        &end,
        const_cast<char*>(&val)
    };

    auto err = ctx->execute([&args, &start, &end, &val] {
        CUmodule mod;

        auto err = cuModuleLoadData(&mod, cuda_kernels);
        if (unlikely(CUDA_SUCCESS != err)) {
            report_cuda_error("cuModuleLoadData", err);
            return ERROR;
        }

        CUfunction fill;
        err = cuModuleGetFunction(&fill, mod, "_Z21fill_cuda_kernel_charPcPKcc");
        if (unlikely(CUDA_SUCCESS != err)) {
            report_cuda_error("cuModuleGetFunction", err);
            return ERROR;
        }

        err = cuLaunchKernel(fill,
                1024, 1, 1, 16, 1, 1,
                0, CU_STREAM_PER_THREAD, args, nullptr);
        if (unlikely(CUDA_SUCCESS != err)) {
            report_cuda_error("cuLaunch", err);
            return ERROR;
        }

        err = cuCtxSynchronize();
        if (unlikely(CUDA_SUCCESS != err)) {
            report_cuda_error("cuCtxSynchronize", err);
            return ERROR;
        }

        err = cuModuleUnload(mod);
        if (unlikely(CUDA_SUCCESS != err)) {
            report_cuda_error("cuModuleUnload", err);
            return ERROR;
        }

        // verify_fill(start, end, val);

        return SUCCESS;
    });

    if (unlikely(err)) {
        fatal("fill() failed.");
    }
}

}
}
