/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "config.h"
#include "compiler.h"
#include "error.h"
#include "pmi.h"
#include <cstdlib>
#include <cstring>
#include <errno.h>

#if 1 == HAVE_MINIPMI
    #include <mpi.h>
    #ifdef __cplusplus
        extern "C" {
    #endif
    #include <minipmi.h>
    #ifdef __cplusplus
        }
    #endif
#endif


static struct minipmi_context minipmi_ctx = { 0 };

#if 1 == HAVE_MINIPMI
    static int minipmi_ctx_refcount = 0;
#endif

#if 1 == HAVE_MINIPMI
    int linktest_minipmi_context_borrow(struct minipmi_context **pmictx)
    {
        if (0 == (minipmi_ctx_refcount++)) {
                std::memset(&minipmi_ctx, 0, sizeof(struct minipmi_context));

                EXEC_IFFAIL(minipmi_initialize(&minipmi_ctx), error("minipmi_initialize() failed."); return ERROR);
        }

        *pmictx = &minipmi_ctx;

        return SUCCESS;
    }
#else
    int linktest_minipmi_context_borrow(struct minipmi_context **pmictx)
    {
        *pmictx = &minipmi_ctx;

        return -EINVAL;
    }
#endif

#if 1 == HAVE_MINIPMI
    int linktest_minipmi_context_return(struct minipmi_context *pmictx)
    {
        if (unlikely(pmictx != &minipmi_ctx)) {
                error("Invalid argument.");
                return ERROR;
        }

        if (0 == (-minipmi_ctx_refcount)) {
                EXEC_IFFAIL(minipmi_finalize(&minipmi_ctx), error("minipmi_finalize() failed."); return ERROR);
        }

        std::memset(&minipmi_ctx, 0, sizeof(struct minipmi_context));

        return SUCCESS;
    }
#else
    int linktest_minipmi_context_return(struct minipmi_context *pmictx)
    {
        if (unlikely(pmictx != &minipmi_ctx)) {
                error("Invalid argument.");
                return ERROR;
        }

        return -EINVAL;
    }
#endif
