/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "serializer.h"
#include "error.h"
#include <numeric>
#include <cstring>

std::size_t Serializer::getSize() const {
    return std::accumulate(bytes.begin(), bytes.end(), std::size_t(0));
}

void Serializer::write(char * dst, [[maybe_unused]]const int rank) const {
    auto srcIt = sources.begin();
    auto numBytesIt = bytes.begin();
    auto descriptionsIt = descriptions.begin();
    for(; srcIt != sources.end(); ++srcIt, ++numBytesIt, ++descriptionsIt){
        std::memcpy(dst, *srcIt, *numBytesIt);
        dst += *numBytesIt;
        #ifdef DEBUG_OUTPUT
            debug("[%3d] Collected %32s: %3lu B", rank, descriptionsIt->c_str(), *numBytesIt);
        #endif
    }
}

void Serializer::read(char * src, [[maybe_unused]]const int rank) const {
    auto dstIt = sources.begin();
    auto numBytesIt = bytes.begin();
    auto descriptionsIt = descriptions.begin();
    for(; dstIt != sources.end(); ++dstIt, ++numBytesIt, ++descriptionsIt){
        std::memcpy(*dstIt, src, *numBytesIt);
        src += *numBytesIt;
        #ifdef DEBUG_OUTPUT
            debug("[%3d] Distributed %32s: %3lu B", rank, descriptionsIt->c_str(), *numBytesIt);
        #endif
    }
}