/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "stats.h"
#include "compiler.h"
#include "error.h"
#include "cmdline.h"
#include "vcluster.h"
#include <cstdlib>
#include <cstring>
#include <ctime>

#define LARGE_FLOAT_NUM (1e20)

void LinktestStats::initialize(VirtualCluster *cl, const struct linktest_args *args) {
    resize(cl->size(), cl->rank(), args->max_stest);
    resetToDefaultValues(cl->rank());
}

void LinktestStats::resize(std::size_t numTasks, std::size_t myRank, std::size_t numRetest) {
    ptimings.resize(numTasks,NO_TIME);
    accesspattern.resize(numTasks, NO_STEP);
    if (myRank == 0) {
        stimings_new.resize(numRetest,NO_TIME);
        stimings_old.resize(numRetest,NO_TIME);
        sfrom.resize(numRetest);
        sto.resize(numRetest);
    }
}

void LinktestStats::resetToDefaultValues(std::size_t myRank){
    avg_time_all =  0.0;
    min_time_all =  LARGE_FLOAT_NUM;
    max_time_all = -LARGE_FLOAT_NUM;
    a2a_avg_time_all =  0.0;
    a2a_min_time_all =  LARGE_FLOAT_NUM;
    a2a_max_time_all = -LARGE_FLOAT_NUM;

    std::fill(ptimings.begin(), ptimings.end(),NO_TIME);
    std::fill(accesspattern.begin(), accesspattern.end(), NO_STEP);
    if (myRank == 0) {
        std::fill(stimings_new.begin(), stimings_new.end(),NO_TIME);
        std::fill(stimings_old.begin(), stimings_old.end(),NO_TIME);
        std::fill(sfrom.begin(), sfrom.end(), NO_RANK);
        std::fill(sto.begin(), sto.end(), NO_RANK);
    }
}

void LinktestStats::setStartTimeString(){
    std::time_t rawtime;
    time(&rawtime);
    const struct tm* const timeinfo = gmtime(&rawtime);
    strftime(startTimeString,LINKTEST_OUTPUT_DATETIME_LENGTH_,"%d/%m/%Y %H:%M:%S UTC+0",timeinfo);
}

void LinktestStats::setEndTimeString(){
    std::time_t rawtime;
    time(&rawtime);
    const struct tm* const timeinfo = gmtime(&rawtime);
    strftime(endTimeString,LINKTEST_OUTPUT_DATETIME_LENGTH_,"%d/%m/%Y %H:%M:%S UTC+0",timeinfo);}
