/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_STATS_H
#define LINKTEST_STATS_H
#include "config.h"
#include <vector>
#include <limits>

class VirtualCluster;
struct linktest_args;

/* Class used to store the gathered statistics.
 */
class LinktestStats {
public:    
    const double   NO_TIME = std::numeric_limits<double>::quiet_NaN();
    const uint64_t NO_STEP = 0; //Zero indicates two tasks in the communication matrix ("Access Pattern") never communicated.
    const uint64_t NO_RANK = std::numeric_limits<uint64_t>::max();

    /* point-to-point ping-pong timings */
    double  avg_time_all;
    double  min_time_all;
    double  max_time_all;

    /* all-to-all timings */
    double  a2a_time;
    double  a2a_avg_time_all;
    double  a2a_min_time_all;
    double  a2a_max_time_all;

    /* individual timings for ping-pong */
    std::vector<double> ptimings;

    /* results from slow pairs tests */
    std::vector<double> stimings_new;
    std::vector<double> stimings_old;
    std::vector<uint64_t> sfrom;
    std::vector<uint64_t> sto;

    /* Mapping: TaskID -> StepID */
    std::vector<uint64_t> accesspattern;

    /* Start & End Time Strings */
    char startTimeString[LINKTEST_OUTPUT_DATETIME_LENGTH_];
    char endTimeString[  LINKTEST_OUTPUT_DATETIME_LENGTH_];

    void initialize(VirtualCluster* cl, const struct linktest_args *args);
    void resize(std::size_t numTasks, std::size_t myRank, std::size_t numRetest);
    void resetToDefaultValues(std::size_t myRank);
    void setStartTimeString();
    void setEndTimeString();
};
#endif
