/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "stopwatch.h"

void Stopwatch::start() {
    reset(),
    resume();
}
void Stopwatch::stop() {
    totalDuration += duration_t( walltime() - resumeTime );
}
void Stopwatch::resume() {
    resumeTime = walltime();
}
void Stopwatch::reset() {
    totalDuration = duration_t::zero();
}
duration_t Stopwatch::getDuration() {
    return totalDuration;
}

class NoopWatch : public StopwatchI {
    public:
    void start() override {}
    void stop() override {}
    void resume() override {}
    void reset() override {}
    duration_t getDuration() override { return duration_t::min(); }

    NoopWatch()                                     = default;
    NoopWatch(const NoopWatch& copyFrom)            = default;
    NoopWatch& operator=(const NoopWatch& copyFrom) = default;
    NoopWatch(NoopWatch &&)                         = default;
    NoopWatch& operator=(NoopWatch &&)              = default;
};

namespace Stopwatchfactory {

    std::unique_ptr<StopwatchI> getRootWatch(int myrank) {
        return getRankWatch(myrank, 0);
    }

    std::unique_ptr<StopwatchI> getRankWatch(int myrank, int watchrank) {
        if(myrank == watchrank) {
            return std::make_unique<Stopwatch>();
        }
        return std::make_unique<NoopWatch>();
    }
}