/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_STOPWATCH
#define LINKTEST_STOPWATCH

#include "timing.h"
#include <memory>

class StopwatchI {
    public:
    virtual void start() = 0;
    virtual void stop() = 0;
    virtual void resume() = 0;
    virtual void reset() = 0;
    virtual duration_t getDuration() = 0;

    virtual ~StopwatchI()                             = default;
    StopwatchI()                                      = default;
    StopwatchI(const StopwatchI& copyFrom)            = default;
    StopwatchI& operator=(const StopwatchI& copyFrom) = default;
    StopwatchI(StopwatchI &&)                         = default;
    StopwatchI& operator=(StopwatchI &&)              = default;
};

class Stopwatch : public StopwatchI {
    public:
    void start() override;
    void stop() override;
    void resume() override;
    void reset() override;
    duration_t getDuration() override;

    ~Stopwatch() override                           = default;
    Stopwatch()                                     = default;
    Stopwatch(const Stopwatch& copyFrom)            = default;
    Stopwatch& operator=(const Stopwatch& copyFrom) = default;
    Stopwatch(Stopwatch &&)                         = default;
    Stopwatch& operator=(Stopwatch &&)              = default;

    private:
    timePoint_t resumeTime = 0;
    duration_t totalDuration = duration_t::zero();
};

namespace Stopwatchfactory {
    std::unique_ptr<StopwatchI> getRootWatch(int rank);
    std::unique_ptr<StopwatchI> getRankWatch(int myrank, int watchrank);
}

#endif