/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_TIMING_H
#define LINKTEST_TIMING_H

#include <chrono>
#include <string>

#if 1 == USE_MPI_WTIME
    #include <mpi.h>
#endif


using timePoint_t = double;
using duration_t = std::chrono::duration<timePoint_t>;

/* Return the walltime in seconds relative to an arbitrary
 * implementation-dependent reference point. The actual
 * precision is system- and implementation-dependent.
 *
 * If the preprocessor flag USE_MPI_WTIME is set walltime
 * is implemented via MPI_Wtime. We use a macro here to
 * avoid any call overhead.
 */
#if 1 == USE_MPI_WTIME
    #define walltime MPI_Wtime
#else
    static inline timePoint_t walltime()
    {
        return duration_t( std::chrono::steady_clock::now().time_since_epoch() ).count();
    }
#endif

void printTimingIfRoot(int rank, std::string step_name, duration_t seconds);

#endif
