/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_UTIL_H
#define LINKTEST_UTIL_H

#include <netinet/ip.h>
#include <ostream>
#include <vector>


class VirtualCluster;
struct linktest_args;

/* Print output from all processing elements such that they appear in order.
 * len must be the same on all processing elements.
 *
 * Internally this is implemented as a gather operation and all strings are
 * printed on a single processing element.
 *
 * In the interest of efficiency, collective_print() is allowed to overwrite
 * the input string!
 */
int collective_print(VirtualCluster* cl, char* str, int len);

/* Latency to bandwidth conversion (seconds to MiB).
 */
double lat_to_bw(double time, const struct linktest_args* args);


template < class T >
std::ostream& operator<< (std::ostream& os, const std::vector<T>& v);
std::ostream& operator<< (std::ostream& os, const std::vector<int>& v); 

namespace linktest
{
using IpAddr = struct sockaddr_in;
}

#endif

template < typename T, std::enable_if_t<std::is_integral<T>::value && std::is_unsigned<T>::value>* = nullptr >
unsigned int number_of_digits_including_minus(T number){
    unsigned int digits = number<0 ? 2 : 1;
    while (number /= 10) digits++;
    return digits;
}
template < typename T, std::enable_if_t<std::is_integral<T>::value && !std::is_unsigned<T>::value>* = nullptr >
unsigned int number_of_digits_including_minus(T number){
    unsigned int digits = 1;
    while (number /= 10) digits++;
    return digits;
}