/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_VCLUSTER_CUDA_H
#define LINKTEST_VCLUSTER_CUDA_H

#include "vcluster.h"

#include <memory>

namespace linktest{
	namespace cuda{
		class VirtualClusterImpl;
	}
}

/* VirtualCluster implementation utilizing CUDA memory operations for data
 * transfer within a node.
 */
class VirtualClusterCUDA : public VirtualClusterWithHelper{

public:
    VirtualClusterCUDA(const std::string& name);

private:
    std::unique_ptr<linktest::cuda::VirtualClusterImpl> impl_;

public:
    int init() override;
    int finalize() override;

    int kpingpong(const int from, const int to, MemoryBuffer& buf,
                          const int num_msg, double* const timing) override;

    int kUniDir(const int from,const int to,
                        MemoryBuffer& buf1, MemoryBuffer& buf2,
                        const int num_msg, double* const timing,
                        const bool doBarrier) override;
    int kUniDirMultiBuf(const int from, const int to,
                                MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                const int num_msg, double* const timing,
                                const bool doBarrier) override;
    int kUniDirLimitedMultiBuf(const int from, const int to,
                                       MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                       const int num_msg, double* const timing,
                                       const bool doBarrier) override;

    int kbipingpong(const int from, const int to,
                            MemoryBuffer& buf1, MemoryBuffer& buf2,
                            const int num_msg, double* const timing) override;

    int write_parallel(const linktest_args* args, const std::vector<LinktestStats>& statsVec) override;
    int write_funnelled(const linktest_args* args, const std::vector<LinktestStats>& statsVec) override;

};
#endif
