/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_VCLUSTER_HELPER_H
#define LINKTEST_VCLUSTER_HELPER_H

#include "compiler.h"
#include <functional>
#include <string>

class VirtualCluster;
class MemoryBuffer;

struct minipmi_context;

std::string determineHostname();

int determineCPUID();


/* Generic barrier implementation based on point-to-point send/receive
 * operations.
 *
 * vcluster_helper_barrier can be used as a generic implementation of
 * the barrier operation operation when no specialized functionality
 * is available or desirable.
 */
int vcluster_helper_barrier(VirtualCluster *cl);

/* Broadcast implementation based on point-to-point send/receive
 * operations.
 *
 * vcluster_helper_bcast can be used as a generic implementation of
 * the bcast operation when no specialized functionality is
 * available or desirable.
 */
int vcluster_helper_bcast(VirtualCluster *cl, int root,
                          MemoryBuffer& buf);

/* Gather operation based on point-to-point send/receive
 * operations.
 *
 * vcluster_helper_gather can be used as a generic implementation of
 * the gather operation when no specialized functionality is
 * available or desirable.
 */
int vcluster_helper_gather(VirtualCluster *cl, int root,
                          MemoryBuffer& dst, MemoryBuffer& src);

/* Scatter operation based on point-to-point send/receive
 * operations.
 *
 * vcluster_helper_scatter can be used as a generic implementation of
 * the scatter operation when no specialized functionality is
 * available or desirable.
 */
int vcluster_helper_scatter(VirtualCluster *cl, int root,
                          MemoryBuffer& dst, MemoryBuffer& src);

/* Reduction operation based on point-to-point send/receive
 * operations.
 *
 * vcluster_helper_reduce can be used as a generic implementation of
 * the reduce operation when no specialized functionality is
 * available or desirable.
 */
int vcluster_helper_reduce(VirtualCluster *cl, int root,
                          MemoryBuffer& dst, MemoryBuffer& src, int op);

/* `connect_all_to_all_via_ring()` establishes connections between each pair
 * of nodes using a ring layout of ranks.
 * `connect(int other)` is used to connect to the `other` rank in a client
 * role. `accept(int other)` is used to connect to `other` in a server role.
 */
int connect_all_to_all_via_ring(int size, int rank,
                                std::function<int(int)> connect,
                                std::function<int(int)> accept);

#endif
