/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_VCLUSTER_MPI_H
#define LINKTEST_VCLUSTER_MPI_H

#include "vcluster.h"
#include <mpi.h>


// VirtualCluster implementation based on a MPI 2.0 implementation
class VirtualClusterMPI : public VirtualCluster
{

public:
    VirtualClusterMPI(const std::string& name);

private:
    MPI_Comm world_;

public:
    int init() override;
    int finalize() override;

    int rank() override;
    int size() override;

    int send(int dst, MemoryBuffer& buf) override;
    int recv(int src, MemoryBuffer& buf) override;

    int barrier() override;
    int bcast(int root, MemoryBuffer& buf) override;
    int gather(int root, MemoryBuffer& dst, MemoryBuffer& src) override;
    int reduce(int root, MemoryBuffer& dst, MemoryBuffer& , int op) override;

    int kalltoall(MemoryBuffer& buffer, const int msg_len, const int num_msg, double* const timing) override;

    int kpingpong(int from, int to, MemoryBuffer& buf, 
                          int num_msg, double* timing) override;

    int kUniDir(const int from, const int to,
                        MemoryBuffer& buf1, MemoryBuffer& buf2,
                        const int num_msg, double* const timing,
                        const bool doBarrier) override;
    int kUniDirMultiBuf(const int from,const int to,
                                MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                const int num_msg, double* const timing,
                                const bool doBarrier) override;
    int kUniDirLimitedMultiBuf(const int from,const int to,
                                       MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                       const int num_msg, double* const timing,
                                       const bool doBarrier) override;

    int kbipingpong(const int from, const int to,
                            MemoryBuffer& buf1, MemoryBuffer& buf2,
                            const int num_msg, double* const timing) override;

    int write_parallel(const linktest_args* args, const std::vector<LinktestStats>& statsVec) override;

};

#endif
