/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_VCLUSTER_PORTALS_H
#define LINKTEST_VCLUSTER_PORTALS_H

extern "C" {
#include <portals4.h>
}
#include <vector>
#include "vcluster.h"

// VirtualCluster implementation based on a Portals 4
class VirtualClusterPortals : public VirtualClusterWithHelper
{

public:
    static constexpr char NAME[] = "portals";
    VirtualClusterPortals();
    int init() override;
    int finalize() override;

    int kpingpong(const int from, const int to, MemoryBuffer& buf, 
                          const int num_msg, double* const timing) override;

    int kUniDir(const int from, const int to,
                        MemoryBuffer& buf1, MemoryBuffer& buf2,
                        const int num_msg, double* const timing,
                        const bool doBarrier) override;
                        
    int kUniDirMultiBuf(const int from,const int to,
                                MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                const int num_msg, double* const timing,
                                const bool doBarrier) override {
        throw("Not Implemented"); // Use PTL_ME_MANAGE_LOCAL
    };
    int kUniDirLimitedMultiBuf(const int from,const int to,
                                       MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                       const int num_msg, double* const timing,
                                       const bool doBarrier) override {
        throw("Not Implemented");
    };

    int kbipingpong(const int from, const int to,
                            MemoryBuffer& buf1, MemoryBuffer& buf2,
                            const int num_msg, double* const timing) override;

private:
    // matching (send/recv) Network Interface (ni)
    ptl_ni_limits_t mni_limits_desired;
    ptl_ni_limits_t mni_limits_actual;
    ptl_handle_ni_t mni_handle;

    /** @brief Portal Table (PT) Index */
    ptl_pt_index_t  pt_index;
    /** @brief PT Event Queue Handle  */
    ptl_handle_eq_t pt_eq_handle;

    const ptl_match_bits_t MATCH_BITS = 1; // TODO when/how to use these?
    const ptl_match_bits_t IGNORE_BITS = ~0; // ignore all bits

    // Sender
    /** @brief Memory Descriptor (MD) */
    ptl_md_t md;
    /** @brief MD Handle */
    ptl_handle_md_t md_handle;
    /** @brief Send Counter */
    ptl_ct_event_t send_ct;
    /** @brief Prepare portals data structures on sender side (MD) */
    void prepareSendStructs(const MemoryBuffer& buf);
    /** @brief Read current send (MD) counter value */
    ptl_size_t getSendCounter();
    /** @brief Free portals data structures on sender side (MD) */
    void releaseSendStructs();

    // Receiver
    /** @brief Match List Entry (ME) */
    ptl_me_t  me;
    /** @brief ME Handle */
    ptl_handle_me_t me_handle;
    /** @brief Receive Counter */
    ptl_ct_event_t recv_ct;
    /** @brief Prepare portals data structures on receiver side (ME) */
    void prepareRecvStructs(const MemoryBuffer& buf);
    /** @brief Read current recv (ME) counter value */
    ptl_size_t getRecvCounter();
    /** @brief Free portals data structures on receiver side (ME) */
    void releaseRecvStructs();

    void sendMessages(const int to, MemoryBuffer& buf, const unsigned long num_msg, const unsigned long counter_start);
    void recvMessages(const unsigned long num_msg, const unsigned long counter_start);

    std::vector<ptl_process_t> getPhysicalFromRank();
};

#endif
