/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_VCLUSTER_TCP_H
#define LINKTEST_VCLUSTER_TCP_H

#include "vcluster.h"
#include <string>

struct minipmi_context;

// VirtualCluster implementation based on TCP sockets
class VirtualClusterTCP : public VirtualCluster
{

public:
    VirtualClusterTCP(const std::string& name);

private:
    int                     rank_;
    int                     size_;

    int                     conn_timeout_;
    int                     listenfd_;
    int*                    socketfds_;

    minipmi_context*        pmi_;

public:
    int init() override;
    int finalize() override;

    int rank() override;
    int size() override;

    int send(int dst, MemoryBuffer& buf) override;
    int recv(int src, MemoryBuffer& buf) override;

private:
    int     read_tcp_environ_rank_and_size();

private:
    int     publish_addr_via_pmi_kvs(int sock, int rank);
    int     read_ipaddr_from_pmi_kvs(int rank, char val[64]);
    int     read_port_from_pmi_kvs(int rank, int *port);

private:
    const char*     read_environ_ipaddr(int rank);
    int             read_environ_port(int rank);
    int             read_environ_sockaddr(int rank, struct sockaddr_in *addr);

private:
    int     open_listen_socket();
    int     accept_socket(int rank, int other);
    int     connect_socket(int rank, int other, int timeout);
    int     connect_to_all();

private:
    void    disable_nagles_algorithm();

private:
    int     disconnect_from_all();

};

#endif
