/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_VCLUSTER_UCP_H
#define LINKTEST_VCLUSTER_UCP_H

#include "vcluster.h"
#include <memory>
#include <ucp/api/ucp.h>

namespace ucp
{

class VirtualClusterImpl;

}

// VirtualCluster implementation utilizing the UCP API.
class VirtualClusterUCP : public VirtualClusterWithHelper
{

public:
    VirtualClusterUCP(const std::string& name);

private:
    std::unique_ptr<ucp::VirtualClusterImpl>        impl_;

public:
    int init() override;
    int finalize() override;

    int kpingpong(const int from, const int to, MemoryBuffer& buf, 
                          const int num_msg, double* const timing) override;

    int kUniDir(const int from, const int to,
                        MemoryBuffer& buf1, MemoryBuffer& buf2,
                        const int num_msg, double* const timing,
                        const bool doBarrier) override;
    int kUniDirMultiBuf(const int from,const int to,
                                MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                const int num_msg, double* const timing,
                                const bool doBarrier) override;
    int kUniDirLimitedMultiBuf(const int from,const int to,
                                       MemoryBufferMulti& buf_multi, MemoryBuffer& buf2,
                                       const int num_msg, double* const timing,
                                       const bool doBarrier) override;

    int kbipingpong(const int from, const int to,
                            MemoryBuffer& buf1, MemoryBuffer& buf2,
                            const int num_msg, double* const timing) override;

private:
    ucs_status_t kpingpong_isend(int other, MemoryBuffer& buf, int tag0);
    ucs_status_t kpingpong_irecv(int other, MemoryBuffer& buf, int tag0);

    int kpingpong_send(int other, MemoryBuffer& buf, int tag0);
    int kpingpong_recv(int other, MemoryBuffer& buf, int tag0);

    int kpingong_waitall(ucs_status_t ok0, ucs_status_t ok1);

};

#endif
