/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#define PY_ARRAY_UNIQUE_SYMBOL linktest_ARRAY_API

#include <linktest.h>
#include <linktest_helper.h>
#include <linktest_2_0_0.h>
#include <linktest_2_1_1.h>
#include <linktest_2_1_2.h>
#include <linktest_2_1_4.h>
#include <linktest_2_1_5.h>
#include <linktest_2_1_6.h>
#include <linktest_2_1_7.h>
#include <linktest_2_1_8.h>
#include <linktest_2_1_9.h>
#include <linktest_2_1_10.h>
#include <linktest_2_1_11.h>
#include <linktest_2_1_12.h>
#include <linktest_2_1_13.h>
#include <linktest_2_1_14.h>
#include <linktest_2_1_15.h>
#include <linktest_2_1_16.h>
#include <linktest_2_1_17.h>
#include <linktest_2_1_18.h>
#include <linktest_2_1_19.h>

/* TODO: Properly handle all possible errors.
 *       Current plan is only to handle errors
 *       as they are encountered and then
 *       implement a safe guard against them
 *       There are too many possible python
 *       related errors to be able to handle
 *       them all individually.
 *       POSTPONED
 */
/* TODO: Properly handle char conversions
 *       from the local encoding of the
 *       saving system to UTF-8 on the
 *       reading system. This will be done
 *       if problems are encountered.
 *       POSTPONED
 */

/*****************/
/* Main Function */
/*****************/
static PyObject* readSION(PyObject *self, PyObject *args){
	int swap, err;
	Version version;
	char* path;
	PyObject* pyLinkTestInfo=NULL;
	PyObject* pyLinkTestData=NULL;
	struct SionFile fh;
//	struct LinkTestInfo linkTestInfo;

	/* Parse Arguments */
	if (!PyArg_ParseTuple(args, "s", &path)) {
		PyErr_SetString(PyExc_RuntimeError, "Failed to parse arguments.");
		return NULL;
	}

	/* Open SION File For Reading */
	openSionFile(path, &fh, &swap);
	if (UNLIKELY(-1 == fh.sid)) {
		/* openSionFile() set the exception exception error string. */
		return NULL;
	}
	sion_seek(fh.sid, 0, 0, 0); //TODO: Is this necessary?

	/* Check The LinkTest Identifier At File Start */
	err = checkLinkTestIdentifier(&fh, swap);
	if (UNLIKELY(err)) {
		/* checkLinkTestIdentifier() sets the exception error string. */
		return NULL;
	}

	/* Determine Version */
	err = getVersion(&fh, version, swap);
	if (UNLIKELY(err)) {
		/* getVersion() sets the exception error string. */
		return NULL;
	}

	/* Switch Reading Based On Version */
	switch(version[0]){
		case 0:
			PyErr_SetString(PyExc_RuntimeError,"LinkTest versions 0.?.? are not supported!");
			err=1;
			break;
		case 1:
			PyErr_SetString(PyExc_RuntimeError,"LinkTest versions 1.?.? are not supported!");
			err=1;
			break;
		case 2:
			switch(version[1]){
				case 0:
					if(version[2]==0){
						err=readImport_2_0_0(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
					}else{
						PyErr_SetString(PyExc_RuntimeError,"LinkTest versions 2.0.X, where X!=0, are not official!");
						err=1;
					}
					break;
				case 1:
					switch(version[2]){
						case  0:
							PyErr_SetString(PyExc_RuntimeError,"LinkTest version 2.0.0 is not supported!");
							err=1;
							break;
						case  1:
							err=readImport_2_1_1(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  2:
							err=readImport_2_1_2(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  3:
							PyErr_SetString(PyExc_RuntimeError,"LinkTest version 2.1.3 is not supported!");
							err=1;
							break;
						case  4:
							err=readImport_2_1_4(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  5:
							err=readImport_2_1_5(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  6:
							err=readImport_2_1_6(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  7:
							err=readImport_2_1_7(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  8:
							err=readImport_2_1_8(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case  9:
							err=readImport_2_1_9(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 10:
							err=readImport_2_1_10(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 11:
							err=readImport_2_1_11(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 12:
							err=readImport_2_1_12(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 13:
							err=readImport_2_1_13(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 14:
							err=readImport_2_1_14(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 15:
							err=readImport_2_1_15(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 16:
							err=readImport_2_1_16(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 17:
							err=readImport_2_1_17(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 18:
							err=readImport_2_1_18(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						case 19:
							err=readImport_2_1_19(&fh,swap,version,&pyLinkTestInfo,&pyLinkTestData);
							break;
						default:
							PyErr_SetString(PyExc_RuntimeError, "Unsupported Version!");
							err=1;
					}
					break;
				default:
					PyErr_SetString(PyExc_RuntimeError, "Unsupported Version!");
					err=1;
			}
			break;
		default:
			PyErr_SetString(PyExc_RuntimeError, "Unsupported Version!");
			err=1;
	}

	if(err){ //If an error occurred we return null pointers
		pyLinkTestInfo = NULL;
		pyLinkTestData = NULL;
	}
	return Py_BuildValue("OO", pyLinkTestInfo, pyLinkTestData);
}

static PyMethodDef methods[] ={
	{"readSION", readSION, METH_VARARGS, "Read data from file"},
	{NULL, NULL, 0, NULL}
};

static struct PyModuleDef reader_module_def = {
	PyModuleDef_HEAD_INIT,
	"reader",     // m_name
	"",           // m_doc
	-1,           // m_size
	methods,      // m_methods
	NULL,         // m_reload
	NULL,         // m_traverse
	NULL,         // m_clear
	NULL,         // m_free
};

#if PY_MAJOR_VERSION >= 3
	PyObject* PyInit_reader()
#else
	void initreader()
#endif
{
	PyObject* module;

#if PY_MAJOR_VERSION >= 3
		module = PyModule_Create(&reader_module_def);
		if (module == NULL)
			return NULL;
#else
		module = Py_InitModule("reader", methods);
		if (module == NULL)
			return;
#endif
	import_array();
#if PY_MAJOR_VERSION >= 3
		return module;
#endif
}
